@@ -23,9 +23,12 @@
 #include <arpa/inet.h>
 #include <ctype.h>
 #include <fcntl.h>
+#include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 #include <sys/ioctl.h>
 #include <sys/socket.h>
+#include <unistd.h>
 #include <linux/sockios.h>
 #include <linux/if.h>
 #include <linux/if_bridge.h>
@@ -38,7 +41,7 @@
 
 #include "libblueman.h"
 
-char* get_net_address(char* iface) {
+char* get_net_address(char* iface, int _ioctl) {
 	int sock = socket(AF_INET, SOCK_STREAM, 0);
 	if (sock < 0) {
 		return NULL;
@@ -47,7 +50,7 @@ char* get_net_address(char* iface) {
 	struct ifreq ifr;
 	strncpy(ifr.ifr_name, iface, IFNAMSIZ);
 	
-	int err = ioctl(sock, SIOCGIFADDR, &ifr);
+	int err = ioctl(sock, _ioctl, &ifr);
 	if (err < 0) {
 		close(sock);
 		return NULL;
@@ -56,6 +59,34 @@ char* get_net_address(char* iface) {
 	return inet_ntoa(((struct sockaddr_in*) &ifr.ifr_addr)->sin_addr);
 }
 
+char** get_interface_list() {
+	int sock = socket(AF_INET, SOCK_STREAM, 0);
+	char** rets = NULL;
+	
+	struct ifconf ifc;
+	ifc.ifc_len = 128 * sizeof(struct ifreq);
+	ifc.ifc_req = malloc(ifc.ifc_len);
+	
+	int res = ioctl(sock, SIOCGIFCONF, &ifc);
+	if(res < 0) {
+
+	} else {
+		int size = ifc.ifc_len/sizeof(struct ifreq);
+	
+		rets = malloc((size+1)*sizeof(char*));
+		memset(rets, '\0', (size+1)*sizeof(char*));
+	
+		int i;
+		for(i = 0; i < size; i++) {
+			rets[i] = strdup(ifc.ifc_req[i].ifr_name);
+		}
+	}
+
+	close(sock);
+	free(ifc.ifc_req);
+	return rets;
+}
+
 static inline unsigned long __tv_to_jiffies(const struct timeval *tv)
 {
         unsigned long long jif;
@@ -96,7 +127,7 @@ int _create_bridge(const char* name) {
         args[4] = 0;
         
         memcpy(ifr.ifr_name, name, IFNAMSIZ);
-        ((unsigned long *)(&ifr.ifr_data))[0] = (unsigned long)args;
+        ifr.ifr_data = (char *) &args;
         
 	ioctl(sock, SIOCDEVPRIVATE, &ifr);
 	
@@ -273,7 +304,7 @@ get_rfcomm_list(struct rfcomm_dev_list_req **result)
 	}
 
 	dl = malloc(sizeof(*dl) + RFCOMM_MAX_DEV * sizeof(*di));
-	if(dl == NULL) {
+	if (dl == NULL) {
 		ret = ERR_CANNOT_ALLOCATE;
 		goto out;
 	}
@@ -283,6 +314,7 @@ get_rfcomm_list(struct rfcomm_dev_list_req **result)
 
 	if (ioctl(ctl, RFCOMMGETDEVLIST, (void *) dl) < 0) {
 		ret = ERR_GET_RFCOMM_LIST_FAILED;
+		free(dl);
 		goto out;
 	}
 
@@ -294,6 +326,76 @@ out:
 	return ret;
 }
 
+int create_rfcomm_device(char *local_address, char *remote_address, int channel) {
+    int sk, dev, ret;
+    struct sockaddr_rc laddr, raddr;
+    struct rfcomm_dev_req req;
+
+    sk = socket(AF_BLUETOOTH, SOCK_STREAM, BTPROTO_RFCOMM);
+    if (sk < 0) {
+        ret = ERR_SOCKET_FAILED;
+        goto out;
+    }
+
+    laddr.rc_family = AF_BLUETOOTH;
+    str2ba(local_address, &laddr.rc_bdaddr);
+    laddr.rc_channel = 0;
+
+    if (bind(sk, (struct sockaddr *) &laddr, sizeof(laddr)) < 0) {
+        ret = ERR_BIND_FAILED;
+        goto out;
+    }
+
+    raddr.rc_family = AF_BLUETOOTH;
+    str2ba(remote_address, &raddr.rc_bdaddr);
+    raddr.rc_channel = channel;
+
+    if (connect(sk, (struct sockaddr *) &raddr, sizeof(raddr)) < 0) {
+        ret = ERR_CONNECT_FAILED;
+        goto out;
+    }
+
+    memset(&req, 0, sizeof(req));
+    req.flags = (1 << RFCOMM_REUSE_DLC) | (1 << RFCOMM_RELEASE_ONHUP);
+    bacpy(&req.src, &laddr.rc_bdaddr);
+    bacpy(&req.dst, &raddr.rc_bdaddr);
+    req.channel = raddr.rc_channel;
+    req.dev_id = -1;
+
+    dev = ioctl(sk, RFCOMMCREATEDEV, &req);
+    if (dev < 0) {
+        ret = ERR_CREATE_DEV_FAILED;
+    } else {
+        ret = dev;
+    }
+
+out:
+    if (sk >= 0)
+        close(sk);
+    return ret;
+}
+
+int release_rfcomm_device(int id) {
+    int sk;
+    struct rfcomm_dev_req req;
+
+    sk = socket(AF_BLUETOOTH, SOCK_RAW, BTPROTO_RFCOMM);
+    if (sk < 0)
+        return ERR_SOCKET_FAILED;
+
+    memset(&req, 0, sizeof(req));
+    req.flags = (1 << RFCOMM_HANGUP_NOW);
+    req.dev_id = id;
+
+    if (ioctl(sk, RFCOMMRELEASEDEV, &req) < 0) {
+        close(sk);
+        return ERR_RELEASE_DEV_FAILED;
+    } else {
+        close(sk);
+        return 0;
+    }
+}
+
 float get_page_timeout(int hdev)
 {
 	struct hci_request rq;
